#include <stdio.h>

int pgcd(int a, int b) {
    int e = (a < 0)?-a:a; // Pour assurer des valeurs positives.
    int f = (b < 0)?-b:b;
    int tmp;
    if (e == 0) return f;
    while (f > 0) {
	tmp   = f;
	f = e % f; // Calcul de r = a % b.
	e = tmp;
    }
    return e;
}

int pgcd_rec(int a, int b) { // Version rcursive.
    int e = (a < 0)?-a:a;
    int f = (b < 0)?-b:b;
    if (f == 0) return e;
    return pgcd_rec(f, e%f);
}

/**
 * Calcul des coefficients de Bezout tels que d = pgcd(a,b) = a.u + b.v.
 */
int bezout(int a, int b, int * u, int * v) {
    int e = (a < 0)?-a:a;
    int f = (b < 0)?-b:b;
    *u = 1;
    *v = 0;
    int u_aux = 0, v_aux = 1;
    int q, r, tmp;
    while (f > 0) {
	q = e/f;
	r = e%f;
	e = f;
	f = r; 
	tmp = u_aux; u_aux = *u - q*u_aux; *u = tmp;
	tmp = v_aux; v_aux = *v - q*v_aux; *v = tmp;
    }
    if (a < 0) *u = -(*u);
    if (b < 0) *v = -(*v);
    return e;
}

int inv_mod(int a, int n, int * res) {
    int d, u, v;
    d = bezout(a, n, &u, &v);
    if (d == 1) {
	*res = (n + u) % n; // u peut tre ngatif et il faut un rsultat dans Z/nZ.
	return 0;
    }
    return 1;
}

int main() 
{
    int a = -14, b = 21;
    int d1 = pgcd(a, b);
    int d2 = pgcd_rec(a, b);
    printf("d1 = %d \t d2 = %d\n", d1, d2);
    return 0;
}
