#include "tools.h"

#define A_SIZE ('z' - 'a' + 1)  // Taille de l'alphabet

int char_to_Z26(char c, int * res) {
	if ( (c >= 'a') && (c <= 'z') ) {// A-t-on bien un lment de [a-z]?
		*res = c - 'a';
		return 0;
	} 
	return 1; // Erreur
}
int Z26_to_char(int i, char * res) {
	if ( (i >= 0) && (i < A_SIZE) ) {
		*res = i + 'a';
		return 0;
	}
	return 1; // Erreur
}

int chiffrement_affine_Z26(int a, int b, int x, int * res) {
	*res = x;
	if ( pgcd(a,A_SIZE) != 1 ) return -1; // Vrification de clef valide.
	if ( (x >= 0) && (x < A_SIZE) ) {
		*res = (a*x + b) % A_SIZE;
		return 0;
	} 
	return 1;
}
int dechiffrement_affine_Z26(int inv_a, int b, int y, int * res) {
	*res = y;
	if ( pgcd(inv_a,A_SIZE) != 1 ) return -1; // Vrification de clef valide.
	if ( (y >= 0) && (y < A_SIZE) ) {
		*res = (inv_a*(A_SIZE + y - b)) % A_SIZE;
		return 0;
	}
	return 1;
}

int chiffrement_affine_Z26_fichier(int a, int b, const char * filein,
				   const char * fileout) {
	FILE *f_in, *f_out;
	char c, res_c;
	int i, res_i;

	if ( pgcd(a,A_SIZE) != 1 ) { // Vrifier la validit de la clef.
		fprintf(stderr, "\nErreur: Clef invalide (a,b)=(%d,%d):"
			" pgcd(a,%d) != 1\n", a, b, A_SIZE);
		return 1;
	}
	if ((f_in = fopen(filein,"r")) == NULL) { // Ouvrir  filein en lecture...
		fprintf(stderr, "\nErreur: Impossible de lire %s\n",filein);
		return 1;
	}
	if ((f_out = fopen(fileout,"w")) == NULL) { // ... et fileout en criture.
		fprintf(stderr, "\nErreur: Impossible d'crire dans %s\n",fileout);
		return 1;
	}
	// Chiffrement du fichier f_in.
	while (1) {
		c = res_c = getc(f_in);
		if ( feof(f_in) ) break;
		if ( ! char_to_Z26(c, &i) ) {
			chiffrement_affine_Z26(a,b,i,&res_i);
			Z26_to_char(res_i, &res_c);
		}
		putc(res_c, f_out);
	}
	// Fermeture des flots de donnes.
	fclose(f_in);
	fclose(f_out);
	return 0;
}
int dechiffrement_affine_Z26_fichier(int a, int b, const char * filein,
				     const char * fileout) {
	FILE *f_in, *f_out;
	char c, res_c;
	int i=0, res_i, inv_a;

	if ( inv_mod(a, A_SIZE, &inv_a) == 1 ) { // Vrifier la validit de la clef.
		fprintf(stderr,"\nErreur: Clef invalide car pgcd(a,%d)!=1\n",
			A_SIZE);
		return 1;
	}
	if ((f_in = fopen(filein,"r")) == NULL) { // Ouvrir filein en lecture...
		fprintf(stderr, "\nErreur: Impossible de lire %s\n",filein);
		return 1;
	}
	if ((f_out = fopen(fileout,"w")) == NULL) { // ... et fileout en criture.
		fprintf(stderr, "\nErreur: Impossible d'crire dans %s\n",fileout);
		return 1;
	}
	// Dchiffrement du fichier f_in.
	while (1) {
		c = res_c = getc(f_in);
		if ( feof(f_in) ) break;
		if ( ! char_to_Z26(c, &i) ) {
			dechiffrement_affine_Z26(inv_a,b,i,&res_i);
			Z26_to_char(res_i, &res_c);
		}
		putc(res_c, f_out);
	}
	// Fermeture des flots de donnes.
	fclose(f_in);
	fclose(f_out);
	return 0;
}

int main()
{
	char * clear_file    = "texte_clair.txt";
	char * cypher_file   = "Out_texte_chiffre.txt";
	char * decypher_file = "Out_texte_dechiffre.txt";
 
	int a = 1, b = 2, inv_a;
	if ( inv_mod(a, A_SIZE, &inv_a) ) {
		fprintf(stderr, "Clef invalide\n");
		return 1;
	}
    
	printf("Chiffrement affine par clef K=(a,b)=(%d,%d)      :",a,b);
	printf(" %s -> %s\n", clear_file, cypher_file);
	if ( chiffrement_affine_Z26_fichier(a,b,clear_file,cypher_file) ) {
		fprintf(stderr, "   Impossible de chiffrer %s\n", clear_file);
		return 1;
	}
    
	printf("\nDchiffrement affine par clef K=(a^-1,b)=(%d,%d) :",inv_a,b);
	printf(" %s -> %s\n", cypher_file, decypher_file);
	if ( dechiffrement_affine_Z26_fichier(inv_a,b,cypher_file,decypher_file ) ) {
		fprintf(stderr, "   Impossible de dechiffrer %s\n", cypher_file);
		return 1;
	}
    
 
    

	return 0;
}
