#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define _POSIX_C_SOURCE 2 /* Ncessaire sur certains Linux. */
#include <netdb.h>
#include <arpa/inet.h>

void print_flags(int f);    // Affichage du champ ai_flags.
void print_family(int af);  // Affichage du champ ai_family.
void print_socktype(int s); // Affichage du champ ai_socktype.
void print_protocol(int p); // Affichage du champ ai_protocol.

void print_addrinfo(struct addrinfo * ai) { // Affichage des infos de la liste ai.
	int i = 0;
	char *ptr = NULL;
	int port = 0;                // Numro de port.
	char addr[INET6_ADDRSTRLEN]; // Adresse rseau.

	while (ai != NULL) {  // Parcours de la liste chane.
		printf("  [%d] ",i++);
		print_family(ai->ai_family);     printf("(%d) ", ai->ai_addrlen);
		print_socktype(ai->ai_socktype); printf(" "); 
		print_protocol(ai->ai_protocol); printf("\n");
		// Rcupration du numro de port et de l'adresse IP.
		switch (ai->ai_family) {
		case AF_INET:  {
			struct sockaddr_in *sin =
				(struct sockaddr_in *) ai->ai_addr;
			ptr = (char *) inet_ntop(AF_INET, &sin->sin_addr,
						 addr, sizeof(addr));
			port = sin->sin_port;
			break;     }
		case AF_INET6: {
			struct sockaddr_in6 *sin6 =
				(struct sockaddr_in6 *) ai->ai_addr;
			ptr = (char *) inet_ntop(AF_INET6, &sin6->sin6_addr,
						 addr, sizeof(addr));
			port = sin6->sin6_port;
			break;     }
		}
		if (ptr != NULL) printf("\t@IP -> %s\t port : %d\n",addr,port);
		if (ai->ai_canonname != NULL)
			printf("\tNom -> %s\n", ai->ai_canonname);
		ai = ai->ai_next; // On passe  l'lment suivant.
	}
}

int main(int argc, char* argv[])
{
	char *hostname = "localhost", *service = NULL;
	struct addrinfo indices, *res;
	int error;

	switch (argc) {
	case 3: service  = argv[2];
	case 2: hostname = argv[1];
	case 1: break;
	default: 
		fprintf(stderr, "Usage: %s [hostname|@IP] [service]\n",argv[0]);
		return EXIT_FAILURE;
	}
	memset(&indices, 0, sizeof(indices)); // Initialisation d'indices.
	indices.ai_flags = AI_CANONNAME;    // On veut une rsolution de nom.
	indices.ai_family = AF_UNSPEC;      // Adresse IPv4 ou IPv6.
	indices.ai_socktype = SOCK_STREAM;  // Socket connecte.
	if ((error = getaddrinfo(hostname, service, &indices, &res))) {
		if (error == EAI_SYSTEM) perror("  [getaddrinfo]");
		fprintf(stderr, "  [getaddrinfo] %s : %s\n", hostname,
			gai_strerror(error));
		return EXIT_FAILURE;
	}
	print_addrinfo(res);
	freeaddrinfo(res); // Ne pas oublier de librer l'espace allou  res! /*@\label{network::getaddrinfo::freeaddrinfo}@*/
	return EXIT_SUCCESS;
}



void print_flags(int f) {
	switch (f) {
	case AI_PASSIVE:     printf("AI_PASSIVE");     break;
	case AI_CANONNAME:   printf("AI_CANONNAME");   break;
	case AI_NUMERICHOST: printf("AI_NUMERICHOST"); break;
	default: printf("%d Unknown",f);
	}
}

void print_family(int af) {
	switch (af) {
	case AF_INET:  printf("AF_INET");  break;
	case AF_INET6: printf("AF_INET6"); break;
	case AF_LOCAL: printf("AF_LOCAL"); break;
	default: printf("%d Uncommon",af);
	}
}

void print_socktype(int s) {
	switch (s) {
	case SOCK_STREAM: printf("SOCK_STREAM"); break;
	case SOCK_DGRAM:  printf("SOCK_DGRAM");  break;
	default: printf("%d Uncommon",s);
	}
}

void print_protocol(int p) {
	switch (p) {
	case IPPROTO_IP:   printf("IPPROTO_IP");   break;
	case IPPROTO_TCP:  printf("IPPROTO_TCP");  break;
	case IPPROTO_UDP:  printf("IPPROTO_UDP");  break;
	case IPPROTO_RAW:  printf("IPPROTO_RAW:"); break;
	case IPPROTO_ICMP: printf("IPPROTO_ICMP"); break;
	default: printf("%d Uncommon",p);
	}
}
