#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#define _POSIX_C_SOURCE 2 /* Ncessaire sur certains Linux. */
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define MAX_SIZE 256

void error(char * msg) { // Affichage d'un message d'erreur et sortie. /*@\label{network::socket_TCP_client::error}@*/
	perror(msg);
	exit(EXIT_FAILURE);
}
/*** Gestion des interactions avec le serveur ***/
void doit(int sockfd, struct sockaddr_in * serv) {
	char requete[MAX_SIZE] = "Luc, je suis ton pre!";
	char reponse[MAX_SIZE + 1];
	size_t taille_req = strlen(requete) + 1;
	size_t reste = taille_req;

	while (reste > 0) { // Envoi de la requte. /*@\label{network::socket_TCP_client::send}@*/
		ssize_t n = send(sockfd, requete + (taille_req - reste), reste, 0);
		if (n < 0) error("[send]");
		printf("Requte envoye  %s:%d : <%s> (%ld octets)\n",
		       inet_ntoa(serv->sin_addr), ntohs(serv->sin_port),
		       requete, (long) n);
		reste -= n;
	}
	ssize_t n = recv(sockfd, reponse, MAX_SIZE-1, 0); // La rponse du serveur./*@\label{network::socket_TCP_client::recv}@*/
	if (n < 0) error("[recv]");
	reponse[n] = '\0'; // Prcaution pour l'affichage.
	printf("Rponse obtenue: <%s> (%ld octets)\n", reponse, (long) n);
}

int main(int argc, char* argv[])
{
	char * servname = "localhost";    // Serveur par dfaut.
	char * port     = "6666";         // Port du service par dfaut.
	struct addrinfo indices, *res;
	int err, sockfd = -1;

	switch (argc) {  // Traitement des arguments de la ligne de commande.
	case 3:  port     = argv[2];
	case 2:  servname = argv[1];
	case 1:  break;
	default: fprintf(stderr, "Usage: %s [servname [port]]\n",argv[0]);
		return EXIT_FAILURE;
	}
        /*** Construction de l'adresse du serveur. ***/
	memset(&indices, 0, sizeof(indices));
	indices.ai_flags    = AI_CANONNAME; // On veut une rsolution de nom.
	indices.ai_family   = AF_INET;      // Adresse IPv4 uniquement. /*@\label{network::socket_TCP_client::diffipv6_1}@*/
	indices.ai_socktype = SOCK_STREAM;  // Type "socket connecte".
	if ((err = getaddrinfo(servname, port, &indices, &res)))  {
		if (err == EAI_SYSTEM) error("[getaddrinfo]");
		fprintf(stderr, "[getaddrinfo] on %s : %s\n", servname,
			gai_strerror(err));
		return EXIT_FAILURE;
	}

	for(; res != NULL; res = res->ai_next) {
		/*** Cration d'une socket. ***/
		sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
		if (sockfd == -1) continue;
		/*** Connexion au serveur. ***/
		if (connect(sockfd, res->ai_addr, res->ai_addrlen)) {  /*@\label{network::socket_TCP_client::connect}@*/
			close(sockfd);
			sockfd = -1;
			continue;
		}
		break; // Si on arrive ici, on a russi  se connecter au serveur.
	}
	if (sockfd  < 0)   error("[socket ou connect]"); 
	doit(sockfd, (struct sockaddr_in *) res->ai_addr); // Gestion des requtes. /*@\label{network::socket_TCP_client::diffipv6_2}@*/
	close(sockfd);     // Fermeture de la socket.
	freeaddrinfo(res); // Ne pas oublier de librer l'espace allou  res!
	return EXIT_SUCCESS;
}

