#include <stdio.h> 
#include <stdlib.h> 
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>    
#include <arpa/inet.h>

#define MAX_SIZE 256
#define NUMPORT 6666     // Le port o les utilisateurs se connecteront.
#define BACKLOG 14       // Nombre maximal de connections acceptes en attente.

void error(char * msg);      // Cf. ligne /*@\ref{network::socket_TCP_client::error} du listing~\ref{network::socket_TCP_client}@*/.
void traitement(int sockfd); // Cf. listing /*@\ref{network::socket_TCP_server::doit}@*/.
int main() 
{
    int listenfd, connfd;            // connfd: socket de travail.
    struct sockaddr_in6 serv_addr;   // Adresse locale du serveur.
    struct sockaddr_in6 client_addr; // Adresse du client qui se connecte.
    char tmp[INET6_ADDRSTRLEN];      // Pour l'affichage de l'adresse IP du client.
    
    /* Cration de l'adresse de socket locale du serveur. */
    memset(&serv_addr, 0, sizeof(serv_addr)); // Remplissage  0.
    serv_addr.sin6_family  = AF_INET6;
    serv_addr.sin6_addr    = in6addr_any;
    serv_addr.sin6_port    = htons(NUMPORT);    
    /* Cration d'une socket. */ 
    if ( (listenfd = socket(PF_INET6, SOCK_STREAM, 0)) < 0) 
	error("[socket]");
    int on = 1,
	rc = setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, 
			(void *)&on, sizeof(on)); 
    if (rc < 0) error("[setsockopt(SO_REUSEADDR)]");
    /* Attachement de la socket. */
    if ( bind(listenfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) 
	error("[bind]");
    /* On en fait une socket d'coute. */ 
    if ( listen(listenfd, BACKLOG) < 0) error("[listen]"); 
    if (inet_ntop(AF_INET6, &serv_addr, tmp, INET6_ADDRSTRLEN))
    printf("Serveur: en coute  l'adresse %s sur le port %d\n",
	   tmp, ntohs(serv_addr.sin6_port));
    while(1) { // Boucle principale pour accept().
	socklen_t addr_len = sizeof(client_addr);
	connfd = accept(listenfd, (struct sockaddr *) &client_addr, &addr_len); 
	if (connfd < 0) {
	    perror("[accept]");
	    continue;
	}
	if (inet_ntop(AF_INET6, &client_addr, tmp, INET6_ADDRSTRLEN))
	    printf("Serveur: connexion accepte avec %s (port %d)\n", 
		   tmp, ntohs(client_addr.sin6_port));
 // ... Le reste est inchang.

	/* Gestion concurrente des connexions accepte */
	if (! fork()) { // Creation d'un processus fils /*@\label{network::socket_TCP_server::fork}@*/
	    close(listenfd);    // fermeture de la socket d'coute pour le fils
	    traitement(connfd); // Traitement de la requte
	    exit(EXIT_SUCCESS); // sortie du processus fils
	}
	close(connfd); // le pre ferme la socket de travail
	while( waitpid( -1, NULL, WNOHANG) > 0); // Nettoyage des processus fils /*@\label{network::socket_TCP_server::waitpid}@*/
    }
    /* Fermeture de la socket */
    close(listenfd);     
    return EXIT_SUCCESS;
}

void error(char * msg) {
    perror(msg);
    exit(EXIT_FAILURE);
}

/**
 * Traite une connexion accepte par envoi/reception de donnes
 */
void traitement(int sockfd) {
    char requete[MAX_SIZE], reponse[MAX_SIZE] = "Nonnnnnnnnn !";
    size_t  answer_size = strlen(reponse)+1;

    ssize_t n = recv(sockfd, requete, MAX_SIZE-1, 0); // Reception
    if (n < 0) error("[recv]");
    requete[n] = '\0'; // Prcaution pour l'affichage
    printf(" - Requte reue: <%s> (%ld octets reus)\n", requete, (long) n);
    
    n = send(sockfd, reponse, answer_size, 0); // Emission de la rponse
    if (n != answer_size ) error("[send]");
    printf(" - Rponse mise: <%s> (%ld octets envoys)\n", reponse, (long) n);
}
