#include <stdio.h> 
#include <stdlib.h> 
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>    

#define MAX_SIZE 256
#define GROUP   "239.15.10.06"  // Format: 239.jj.mm.yy  
#define NUMPORT 6666

void error(char * msg);
int main() 
{
    int sockfd = socket(PF_INET, SOCK_DGRAM, 0); // initialisation classique
    if (sockfd < 0) error("[socket]");

    struct sockaddr_in ip_mcast; // adresse de multicast du groupe (@IP+port)
    memset(&ip_mcast, 0, sizeof(ip_mcast));
    ip_mcast.sin_family      = AF_INET;
    if (inet_pton(AF_INET, GROUP, &(ip_mcast.sin_addr)) < 0) error("[inet_pton]");
    ip_mcast.sin_port        = htons(NUMPORT);    
 
    /* pour recevoir des donnes par multicast, il faut :
          1/ s'abonner au groupe
	  2/ attacher la socket au port par bind */
    struct ip_mreq  gr_mcast;  
    gr_mcast.imr_multiaddr.s_addr = ip_mcast.sin_addr.s_addr;
    gr_mcast.imr_interface.s_addr = htons(INADDR_ANY);
    // abonnement de la socket au groupe multicast
    int rc = setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
			&gr_mcast, sizeof(gr_mcast));
    if (rc < 0) error("setsockopt(IP_ADD_MEMBERSHIP)]");
    // Eventuellement, autoris de lier plusieurs socket sur le port
    const int on = 1;
    rc = setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
    if (rc < 0) error("[setsockopt(SO_REUSEADDR)]");

    // Attachement de la socket
    if (bind(sockfd, (struct sockaddr *) &ip_mcast, sizeof(ip_mcast)) < 0)
	error("[bind]");
    
    // On peut maintenant recevoir des donnes multicast
    char msg[MAX_SIZE];
    while (1) {
	ssize_t n = recvfrom(sockfd, msg, MAX_SIZE-1, 0, NULL, NULL);
	if (n < 0) error("[recvfrom]");
	msg[MAX_SIZE-1] = '\0';
	printf("Recv : %s\n",msg);
    }
    close(sockfd);
    return EXIT_SUCCESS;
}

void error(char * msg) {
    perror(msg);
    exit(EXIT_FAILURE);
}
